#!/usr/bin/python
# -*- coding: utf-8 -*-
from SinaWeibo import Weibo

if __name__ == '__main__':
    wb= Weibo("","")
    wb.postMessage("0.2测试1:文本")
    # time.sleep(1)
    # wb.postImage("0.2测试2:一张图片","/Downloads/4.png")
    # time.sleep(1)
    # wb.postImage("0.2测试3:多张图片","/Downloads/4.png","/Downloads/5.jpg")
    #
    # # 我的关注
    # pageNum = 1
    # followList, hasNext = wb.getFollowList(pageNum)
    # print(followList)
    # while hasNext == True:
    #     pageNum = pageNum + 1
    #     followList, hasNext = wb.getFollowList(pageNum)
    #     print(followList)
    #
    # # 我的粉丝
    # pageNum = 1
    # fansList , hasNext = wb.getFansList(pageNum)
    # print(fansList)
    # while hasNext == True:
    #     pageNum = pageNum + 1
    #     fansList, hasNext = wb.getFansList(pageNum)
    #     print(fansList)
    #
    # # 我的微博
    # blogList = wb.getMyBlogList(1)
    # for blog in blogList:
    #     print(blog)





#!/usr/bin/python
# -*- coding: utf-8 -*-
from enum import Enum

class FollowType(Enum):
    ORG = 1
    USER = 2

class Follow(object):
    followType = None
    id = ""
    name = ""
    href = ""

    def __init__(self,followType,id,name,href):
        assert type(followType) == type(FollowType.ORG)

        self.followType = followType
        self.id = id
        self.name = name
        self.href = href

    def __str__(self):
        return "{name:%s,id:%s,href:%s,type:%s}" % (self.name, self.id, self.href, str(self.followType))
    def __repr__(self):
        return "{name:%s,id:%s,href:%s,type:%s}" % (self.name, self.id, self.href, str(self.followType))




#!/usr/bin/python
# -*- coding: utf-8 -*-

class Fans(object):
    id = ""
    href = ""
    name = ""

    def __init__(self, id, name, href):
        self.id = id
        self.name = name
        self.href = href

    def __str__(self):
        return "{name:%s,id:%s,href:%s}" % (self.name, self.id, self.href)

    def __repr__(self):
        return "{name:%s,id:%s,href:%s}" % (self.name, self.id, self.href)



#!/usr/bin/python
# -*- coding: utf-8 -*-

class Blog(object):
    id = ""
    text = ""
    date = ""

    def __init__(self, id, text, date):
        self.id = id
        self.text = text
        self.date = date

    def __str__(self):
        return "{id:%s,text:%s,date:%s}" % (self.id, self.text, self.date)

    def __repr__(self):
        return "{id:%s,text:%s,date:%s}" % (self.id, self.text, self.date)


#!/usr/bin/python
# -*- coding: utf-8 -*-


import binascii
import rsa
import base64
import requests
import json
from bs4 import BeautifulSoup

from .follow import Follow
from .follow import FollowType
from .fans import Fans
from .blog import Blog


class WbUtils(object):
    @staticmethod
    def encrypt_passwd(passwd, pubkey, servertime, nonce):
        key = rsa.PublicKey(int(pubkey, 16), int('10001', 16))
        message = str(servertime) + '\t' + str(nonce) + '\n' + str(passwd)
        passwd = rsa.encrypt(message.encode('utf-8'), key)
        return binascii.b2a_hex(passwd)

    @staticmethod
    def getLoginStructure(logincode, password, pre_login):
        data = {
            'entry': 'weibo',
            'gateway': 1,
            'from': '',
            'savestate': 7,
            'userticket': 1,
            'ssosimplelogin': 1,
            'su': base64.b64encode(requests.utils.quote(logincode).encode('utf-8')),
            'service': 'miniblog',
            'servertime': pre_login['servertime'],
            'nonce': pre_login['nonce'],
            'vsnf': 1,
            'vsnval': '',
            'pwencode': 'rsa2',
            'sp': WbUtils.encrypt_passwd(password, pre_login['pubkey'], pre_login['servertime'], pre_login['nonce']),
            'rsakv': pre_login['rsakv'],
            'encoding': 'UTF-8',
            'prelt': '53',
            'url': 'http://weibo.com/ajaxlogin.php?framelogin=1&callback=parent.si' 'naSSOController.feedBackUrlCallBack',
            'returntype': 'META'
        }
        return data

    def getTextStructure(message):
        data = {
            'location': 'v6_content_home',
            'text': message,
            'appkey': '',
            'style_type': 1,
            'pic_id': '',
            'pdetail': '',
            'rank': 0,
            'rankid': '',
            'module': 'stissue',
            'pub_source': 'main_',
            'pub_type': 'dialog',
            'isPri': 0,
            '_t': 0
        }
        return data

    def getImageStructure(message, pic_id, picNum):
        data = WbUtils.getTextStructure(message)
        data["pic_id"] = pic_id
        if picNum > 0:
            data["updata_img_num"] = picNum
        return data

    def checkResultMessage(resultJson):
        resultFlag = False
        resultObject = json.loads(resultJson)
        try:
            code = resultObject['code']
            msg = resultObject['msg']
            data = resultObject['data']
            if code == '100000':
                resultFlag = True
            return resultFlag, msg, data
        except Exception as e:
            raise e

    def getFMViewObjDict(htmlcontent):
        FMViewDict = {}
        soup = BeautifulSoup(htmlcontent, 'html.parser')
        for FMView in soup.find_all('script'):
            FMViewStr = FMView.get_text().strip()
            if FMViewStr.startswith('FM.view('):
                lastIndex = len(FMViewStr) - 1
                if FMViewStr.endswith(';'):
                    lastIndex = lastIndex - 1
                fmObj = json.loads(FMViewStr[8:lastIndex])
                if list(fmObj).__contains__('domid'):
                    FMViewDict[str(fmObj['domid'])] = fmObj
        return FMViewDict

    def getMyInfo(fmDict):
        follow = 0
        fans = 0
        weibo = 0
        try:
            fmhtml = fmDict.get('v6_pl_rightmod_myinfo')['html']
            soup = BeautifulSoup(fmhtml, 'html.parser')
            for a in soup.find_all("a", attrs={"bpfilter": "page_frame", "class": "S_txt1"}):
                strong = a.find('strong')
                if strong != None:
                    nodeType = strong['node-type']
                    if nodeType == 'follow':
                        follow = strong.get_text().strip()
                    if nodeType == 'fans':
                        fans = strong.get_text().strip()
                    if nodeType == 'weibo':
                        weibo = strong.get_text().strip()
            return follow, fans, weibo
        except Exception as e:
            print("获取用户基本信息异常")
            raise e

    def __getPageCount(soup):
        pageCount = 0
        W_pages = soup.find("div", attrs={"class": "W_pages"})
        pageList = W_pages.find_all("a", attrs={"class": "page S_txt1"})
        if len(pageList) > 0:
            pageCount = pageList[-1].get_text().strip()
        return pageCount

    def getFollowList(fmDict):
        followList = []
        fmhtml = fmDict.get('Pl_Official_RelationMyfollow__93')['html']
        soup = BeautifulSoup(fmhtml, 'html.parser')
        for li in soup.find_all("li", attrs={"class": "member_li S_bg1"}):
            node = li.find("a", attrs={"node-type": "screen_name"})
            if node != None:
                if node.attrs.__contains__('usercard'):
                    followList.append(
                        Follow(FollowType.USER, node['usercard'].split('=')[1], node['title'], node['href']))
                else:
                    followList.append(Follow(FollowType.ORG, "", node['title'], node['href']))

        return followList, int(WbUtils.__getPageCount(soup))

    def getFansList(fmDict):
        fansList = []
        fmhtml = fmDict.get('Pl_Official_RelationFans__88')['html']
        soup = BeautifulSoup(fmhtml, 'html.parser')
        for li in soup.find_all("li", attrs={"class": "follow_item S_line2", "node-type": "userItem"}):
            node = li.find("a", attrs={"class": "S_txt1"})
            fansList.append(Fans(node["title"], node["usercard"].split('&')[0].split('=')[0], node["href"]))
        return fansList, int(WbUtils.__getPageCount(soup))

    def getProfileHtml(paramsStr):
        htmlStr = ""
        if paramsStr.startswith("<script>parent.FM.view("):
            htmlStr = json.loads(paramsStr[23:len(paramsStr) - 11])['html']
        return htmlStr

    def getBlogList(htmlContent):
        blogList = []
        soup = BeautifulSoup(htmlContent, 'html.parser')
        for item in soup.find_all("div", attrs={"action-type": "feed_list_item"}):
            mid = item['mid']
            content = item.find("div", attrs={"node-type": "feed_list_content"})
            item_date = item.find("a", attrs={"node-type": "feed_list_item_date"})
            blogList.append(Blog(mid, content.get_text().strip(), item_date.get_text().strip()))
        return blogList





from setuptools import setup,find_packages
setup(
        name='sinaweibo',
        version='0.4',
        description='SinaWeibo',
        author='xjouyi@163.com',
        author_email='xjouyi@163.com',
        url='https://github.com/XJouYi/SinaWeibo',
        classifiers=[  # Optional
                'Development Status :: 3 - Alpha',
                'Intended Audience :: Developers',
                'Topic :: Software Development :: Build Tools',
                'License :: OSI Approved :: MIT License',

                # Specify the Python versions you support here. In particular, ensure
                # that you indicate whether you support Python 2, Python 3 or both.
                'Programming Language :: Python :: 2',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.4',
                'Programming Language :: Python :: 3.5',
                'Programming Language :: Python :: 3.6',
        ],
        packages=['SinaWeibo'],
        include_package_data=True,
        zip_safe=True,
        install_requires=['requests', 'rsa','beautifulsoup4'],
)


# -*- coding: utf-8 -*-

__doc__ = 'Sian Weibo Client API'
__project__ = 'SinaWeibo'
__version__ = '0.3'

from .weibo import Weibo
from .fans import Fans
from .follow import Follow,FollowType
from .blog import Blog



#!/usr/bin/python
# -*- coding: utf-8 -*-
import base64
import json
import random
import re
import time
import requests
from .utils import WbUtils

WBCLIENT = 'ssologin.js(v1.4.19)'
user_agent = (
    'Mozilla/5.0 (Windows NT 5.1) AppleWebKit/536.11 (KHTML, like Gecko) '
    'Chrome/20.0.1132.57 Safari/536.11'
)


class Weibo(object):

    logincode = ""
    password = ""
    uid = ""
    homeUrl = ""

    def __init__(self,logincode,password):
        self.logincode = logincode
        self.password = password

        self.session = requests.session()
        self.session.headers = {
            "User-Agent": user_agent
        }
        self.__login()
        follow, fans, weibo = self.userInfo()
        print("关注：%s,粉丝:%s,微博:%s"%(follow,fans,weibo))

    def __login(self):
        try:
            resp = self.session.get('https://login.sina.com.cn/sso/prelogin.php?entry=weibo&callback=sinaSSOController.preloginCallBack&su=%s&rsakt=mod&checkpin=1&client=%s' %
                                    (base64.b64encode(self.logincode.encode('utf-8')), WBCLIENT)
            )
            pre_login = json.loads(re.match(r'[^{]+({.+?})', resp.text).group(1))
            resp = self.session.post( 'https://login.sina.com.cn/sso/login.php?client=%s' %
                                      WBCLIENT, data=WbUtils.getLoginStructure(self.logincode,self.password,pre_login)
            )
            crossdomain2 = re.search('(http[s]{0,1}://[a-zA-Z0-9\\.\\-]+\\.([a-zA-Z]{2,4})(:\\d+)?(/[a-zA-Z0-9\\.\\-~!@#$%^&*+?:_/=<>]*)?)|((www.)|[a-zA-Z0-9\\.\\-]+\\.([a-zA-Z]{2,4})(:\\d+)?(/[a-zA-Z0-9\\.\\-~!@#$%^&*+?:_/=<>]*)?)', resp.text).group(1)
            resp = self.session.get(crossdomain2)
            passporturl = re.search("(https://passport[a-zA-Z0-9\\.\\-]+\\.([a-zA-Z]{2,4})(:\\d+)?(/[a-zA-Z0-9\\.\\-~!@#$%^&*+?:_/=<>]*)?)",resp.text.replace('\/','/')).group(0)
            resp = self.session.get(passporturl)
            login_info = json.loads(re.search('\((\{.*\})\)', resp.text).group(1))
            self.uid = login_info["userinfo"]["uniqueid"]
            print("%s 登录成功"%self.logincode)
        except Exception as e:
            print("%s 登录失败"%self.logincode)
            raise e

    def __str__(self):
        return "logincode:%s,password:%s,uid:%s,url:%s"%(self.logincode,self.password,self.uid,self.homeUrl)

    def userInfo(self):
        resp = self.session.get("https://weibo.com/")
        self.homeUrl = resp.url
        self.baseUrl = self.homeUrl[:self.homeUrl.index("home")-1]
        fmDict = WbUtils.getFMViewObjDict(resp.text)
        follow, fans, weibo = WbUtils.getMyInfo(fmDict)
        return follow, fans, weibo

    def __postData(self,data):
        currTime = "%d" % (time.time()*1000)
        self.session.headers["Host"]="weibo.com"
        self.session.headers["Origin"]="https://weibo.com"
        Referer = "https://www.weibo.com/u/%s/home?wvr=5" % self.uid
        self.session.headers["Referer"] = Referer
        resp = self.session.post(
            'https://weibo.com/aj/mblog/add?ajwvr=6&__rnd=%s'%currTime,data = data
        )
        try:
            result, msg, txt= WbUtils.checkResultMessage(resp.content)
            if result == True:
                print("消息发送成功:%s" % data['text'])
            else:
                print("消息发送失败:%s" % msg)
        except Exception as e:
            print("消息发送失败:%s" % data)
            raise e

    def postMessage(self,message):
        self.__postData(WbUtils.getTextStructure(message))

    def uploadPic(self,picPath):
        url = 'weibo.com/u/' + self.uid
        self.session.headers["Referer"] = self.homeUrl
        self.session.headers["Host"] = "picupload.weibo.com"
        self.session.headers["Origin"] = "https://weibo.com"
        resp = self.session.post(
            'https://picupload.weibo.com/interface/pic_upload.php?app=miniblog' +
            '&data=1&url=' + url + '&markpos=1&logo=1&nick=&marks=1&url=' + url +
            '&mime=image/png&ct=' + str(random.random()),
            data=open(picPath, 'rb')
        )
        resultJson = json.loads(re.search('{"code.*', resp.text).group(0))
        return resultJson["data"]["pics"]["pic_1"]["pid"]

    def postImage(self,message,*picPaths):
        picList = []
        for pic in picPaths:
            picList.append(self.uploadPic(pic))
        self.__postData(WbUtils.getImageStructure(message,"|".join(picList),len(picList)))

    def getFollowList(self,pageNum=1):
        url = "https://weibo.com/p/100505%s/myfollow?t=1&cfs=&Pl_Official_RelationMyfollow__93_page=%d#Pl_Official_RelationMyfollow__93"%(self.uid,pageNum)
        resp = self.session.get(url)
        fmDict = WbUtils.getFMViewObjDict(resp.text)
        followList,pageCount = WbUtils.getFollowList(fmDict)
        return followList,pageNum < pageCount

    def getFansList(self,pageNum=1):
        url = "https://weibo.com/%s/fans?cfs=600&relate=fans&t=1&f=1&type=&Pl_Official_RelationFans__88_page=%d#Pl_Official_RelationFans__88"%(self.uid,pageNum)
        resp = self.session.get(url)
        fmDict = WbUtils.getFMViewObjDict(resp.text)
        fansList, pageCount = WbUtils.getFansList(fmDict)
        return fansList, pageNum < pageCount

    def __getMyBlogList(self,pageNum,pagebar = 0):
        url = "https://weibo.com/p/aj/v6/mblog/mbloglist?ajwvr=6&domain=100505&is_search=0&visible=0&is_all=1&is_tag=0&profile_ftype=1" \
            "&pl_name=Pl_Official_MyProfileFeed__21&id=1005051656558815&script_uri=&feed_type=0" \
            "&page=%d&pagebar=%d&pre_page=1&domain_op=100505&__rnd=1516869103198"%(pageNum,pagebar)
        resp = self.session.get(url)
        _flag,msg,data = WbUtils.checkResultMessage(resp.text)
        if _flag:
            contentList = WbUtils.getBlogList(data)
            return contentList

    def getMyBlogList(self,pageNum=1):
        url = self.baseUrl + "/profile?pids=Pl_Official_MyProfileFeed__21&is_search=0&visible=0&is_all=1&is_tag=0&profile_ftype=1&page=%d&ajaxpagelet=1&ajaxpagelet_v6=1&__ref="%(pageNum)
        resp = self.session.get(url)
        htmlStr =  WbUtils.getProfileHtml(resp.text)
        blogList = WbUtils.getBlogList(htmlStr)
        blogList.extend(self.__getMyBlogList(pageNum,0))
        blogList.extend(self.__getMyBlogList(pageNum,1))
        return blogList


